using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Administrator.DataBase;
using Administrator.DataBase.DBModels;

namespace Administrator.Controllers
{
    [ApiController]
    [Route("Administrators/")]
    public class AdminController : ControllerBase
    {
        private readonly AdminDataBaseContext _context;
        private readonly ILogger<AdminController> _logger;
        private readonly IHttpClientFactory _clientFactory;

        public AdminController(AdminDataBaseContext context, ILogger<AdminController> logger, IHttpClientFactory clientFactory)
        {
            _context = context;
            _logger = logger;
            _clientFactory = clientFactory;
        }

        //   
        [HttpPost]
        public async Task<ActionResult<Admin>> CreateAdministrator(Admin administrator)
        {
            if (!await ClientExists(administrator.ClientId))
            {
                return BadRequest("  .");
            }

            _context.Administrators.Add(administrator);
            await _context.SaveChangesAsync();

            return CreatedAtAction(nameof(GetAdministratorById), new { id = administrator.Id }, administrator);
        }

        //  
        [HttpPut("{id}")]
        public async Task<IActionResult> UpdateAdministrator(int id, Admin updatedAdministrator)
        {
            if (!await ClientExists(updatedAdministrator.ClientId))
            {
                return BadRequest("  .");
            }

            if (id != updatedAdministrator.Id)
            {
                return BadRequest();
            }

            _context.Entry(updatedAdministrator).State = EntityState.Modified;

            try
            {
                await _context.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!AdminExists(id))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return NoContent();
        }

        //  
        [HttpDelete("{id}")]
        public async Task<IActionResult> DeleteAdministrator(int id)
        {
            var administrator = await _context.Administrators.FindAsync(id);
            if (administrator == null)
            {
                return NotFound("  ");
            }

            _context.Administrators.Remove(administrator);
            await _context.SaveChangesAsync();

            return NoContent();
        }

        //    
        [HttpGet]
        public async Task<ActionResult<IEnumerable<Admin>>> GetAllAdministrators()
        {
            return await _context.Administrators.ToListAsync();
        }

        //    ID
        [HttpGet("{id}")]
        public async Task<ActionResult<Admin>> GetAdministratorById(int id)
        {
            var administrator = await _context.Administrators.FindAsync(id);

            if (administrator == null)
            {
                return NotFound("  ");
            }

            return administrator;
        }

        private bool AdminExists(int id)
        {
            return _context.Administrators.Any(e => e.Id == id);
        }

        private async Task<bool> ClientExists(int buyerId)
        {
            var client = _clientFactory.CreateClient("BuyerService");
            var response = await client.GetAsync($"Buyers/{buyerId}");
            return response.IsSuccessStatusCode;
        }
    }
}
