using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Logging;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Buyer.DataBase;
using Buyer.DataBase.DBModels;

namespace Buyer.Controllers
{
    [ApiController]
    [Route("Buyers/")]
    public class BuyerController : Controller
    {
        private readonly BuyerDataBaseContext _dbContext;

        public BuyerController(BuyerDataBaseContext dbContext)
        {
            _dbContext = dbContext;
        }

        //    
        [HttpGet]
        public async Task<ActionResult<IEnumerable<Buyers>>> GetAllBuyers()
        {
            return await _dbContext.Buyers.ToListAsync();
        }

        //    ID
        [HttpGet("{id}")]
        public async Task<ActionResult<Buyers>> GetBuyer(int id)
        {
            var Buyer = await _dbContext.Buyers.FindAsync(id);

            if (Buyer == null)
            {
                return NotFound();
            }

            return Buyer;
        }

        //   
        [HttpPost]
        public async Task<ActionResult<Buyers>> CreateBuyer(Buyers Buyer)
        {
            _dbContext.Buyers.Add(Buyer);
            await _dbContext.SaveChangesAsync();

            return CreatedAtAction(nameof(GetBuyer), new { id = Buyer.Id }, Buyer);
        }

        //  
        [HttpPut("{id}")]
        public async Task<IActionResult> UpdateBuyer(int id, Buyers Buyer)
        {
            if (id != Buyer.Id)
            {
                return BadRequest();
            }

            _dbContext.Entry(Buyer).State = EntityState.Modified;

            try
            {
                await _dbContext.SaveChangesAsync();
            }
            catch (DbUpdateConcurrencyException)
            {
                if (!BuyerExists(id))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return NoContent();
        }

        //  
        [HttpDelete("{id}")]
        public async Task<IActionResult> DeleteBuyer(int id)
        {
            var Buyer = await _dbContext.Buyers.FindAsync(id);
            if (Buyer == null)
            {
                return NotFound();
            }

            _dbContext.Buyers.Remove(Buyer);
            await _dbContext.SaveChangesAsync();

            return NoContent();
        }

        private bool BuyerExists(int id)
        {
            return _dbContext.Buyers.Any(e => e.Id == id);
        }
    }
}
